<?php
namespace Teknisa\CADEST\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Ger03800ProdutosPorUnidadeService extends \Teknisa\CADEST\Controllers\TeknisaCrud {

	protected $environment;
	protected $dataSourceManager;
	protected $newCode;
	protected $factory;

    public function __construct(Environment $environment, $dataSourceManager, $newCode, $factory){
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode = $newCode;
        $this->factory = $factory;
    }

    
    public function cadastroAutomaticoControleEstoque($rowObject) {
        $row = (array) $rowObject;
        $dataSourceNameToSave = 'prodfili_produto_ctrlestq';
        $produtos = $this->getProdutos($row['CDFILIAL'], $row['P_CDPRODINI'], $row['P_CDPRODFIN'], NULL);
        $rowsUpdate = array();
        $count = 0;
        foreach ($produtos as $prod) {
            $produto = array_merge($prod, $row);
            unset($produto['__rightActions']);
            unset($produto['__leftActions']);
            unset($produto['P_CDPRODINI']);
            unset($produto['P_CDPRODFIN']);
            $produto = $this->adicionaCampos($produto);
            array_push($rowsUpdate, $this->verificaCamposAlterados($row, $produto));
            $count++;
            if($count >= 500){
                $dataSet = new DataSet($dataSourceNameToSave, $rowsUpdate);
                $this->dataSourceManager->persist($dataSet);
                $rowsUpdate = array();
                $count = 0;
            }
        }
        $dataSet = new DataSet($dataSourceNameToSave, $rowsUpdate);
        $this->dataSourceManager->persist($dataSet);
    }
    
    protected function getProdutos($CDFILIAL, $P_CDPRODINI, $P_CDPRODFIN, $TODOS) {
        $conditions =  array();
        $filterCriteria = new FilterCriteria('prodfili_produto_ctrlestq', $conditions);
        $paramsProdutos = $params = array(
                    ':CDFILIAL'=>$CDFILIAL,
                    ':P_CDPRODFIN'=>$P_CDPRODFIN,
                    ':P_CDPRODINI'=>$P_CDPRODINI,
                    ':TODOS'=>$TODOS
                );
        $produtos = $this->factory->getProdutos($params);
        $prodReturn = array();
        foreach ($produtos as $prod) {
            $produtoComLocal = $this->adicionaCampos($prod);
            array_push($prodReturn, $produtoComLocal);
        }
        return $prodReturn;
    }
    
    protected function verificaCamposAlterados($row, $prod) {
        $prod["IDCNTRESTOQ"] = isset($row["IDCNTRESTOQ"]) ? $row["IDCNTRESTOQ"] : $prod["IDCNTRESTOQ"];
        $prod["NRDIASESTOQ"] = isset($row["NRDIASESTOQ"]) ? $row["NRDIASESTOQ"] : $prod["NRDIASESTOQ"];
        $prod["IDUTCTRLLOTE"] = isset($row["IDUTCTRLLOTE"]) ? $row["IDUTCTRLLOTE"] : $prod["IDUTCTRLLOTE"];
        $prod["IDUTVRFTCONVVAR"] = isset($row["IDUTVRFTCONVVAR"]) ? $row["IDUTVRFTCONVVAR"] : $prod["IDUTVRFTCONVVAR"];
        $prod["IDPRODCOMRASTRO"] = isset($row["IDPRODCOMRASTRO"]) ? $row["IDPRODCOMRASTRO"] : $prod["IDPRODCOMRASTRO"];
        $prod["NRDIASLIMSEGVAL"] = isset($row["NRDIASLIMSEGVAL"]) ? $row["NRDIASLIMSEGVAL"] : $prod["NRDIASLIMSEGVAL"];
        return $prod;
    }
    
    protected function verificaCamposAlteradosProdfili($row, $prod) {
        // var_dump($prod);die;
        
        $prod["IDUTILPESAGEM"]      = isset($row["IDUTILPESAGEM"])      ? $row["IDUTILPESAGEM"]     : $prod["IDUTILPESAGEM"];
        $prod["VRPTOLQTITAF"]       = isset($row["VRPTOLQTITAF"])       ? $row["VRPTOLQTITAF"]      : $prod["VRPTOLQTITAF"];
        $prod["VRPTOLVRITAF"]       = isset($row["VRPTOLVRITAF"])       ? $row["VRPTOLVRITAF"]      : $prod["VRPTOLVRITAF"];
        $prod["VRPTOLAJUSINV"]      = isset($row["VRPTOLAJUSINV"])      ? $row["VRPTOLAJUSINV"]     : $prod["VRPTOLAJUSINV"];
        $prod["VRTXADMPROD"]        = isset($row["VRTXADMPROD"])        ? $row["VRTXADMPROD"]       : $prod["VRTXADMPROD"];
        $prod["IDUTPRORIGOBRIG"]    = isset($row["IDUTPRORIGOBRIG"])    ? $row["IDUTPRORIGOBRIG"]   : $prod["IDUTPRORIGOBRIG"];
        $prod["VRMINEMPENHOAF"]     = isset($row["VRMINEMPENHOAF"])     ? $row["VRMINEMPENHOAF"]    : $prod["VRMINEMPENHOAF"];
        $prod["VRMINEMPENHOOL"]     = isset($row["VRMINEMPENHOOL"])     ? $row["VRMINEMPENHOOL"]    : $prod["VRMINEMPENHOOL"];
        return $prod;
    }
    
    protected function adicionaCampos($row) {
        $nrorg = $this->environment->getNrorgTrab();
		$operador = $this->environment->getCdOperador();
		$currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
		$row['NRORG'] = $nrorg;
		$row['DTINCLUSAO'] = $currentTime;
		$row['DTULTATU'] = $currentTime;
		$row['CDOPERINCLUSAO'] = $operador;
		$row['CDOPERULTATU'] = $operador;
		$row['IDATIVO'] = 'S';
		$row['NRORGINCLUSAO'] = $nrorg;
		$row['NRORGULTATU'] = $nrorg;
		$row['__is_new'] = false;
		return $row;
    }
    
    public function cadastroAutomaticoProdutoFilial($rowObject) {
        $row = (array) $rowObject;
        $dataSourceNameToSave = 'prodporfili_produto';
        $produtos = $this->getProdFili($row['CDFILIAL'], $row['P_CDPRODINI'], $row['P_CDPRODFIN'], NULL);
        $rowsUpdate = array();
        $count = 0;
        foreach ($produtos as $prod) {
            // var_dump($prod);die;
            $produto = array_merge($prod, $row);
            unset($produto['__rightActions']);
            unset($produto['__leftActions']);
            unset($produto['P_CDPRODINI']);
            unset($produto['P_CDPRODFIN']);
            $produto["IDPRODATIV"]= $rowObject["IDPRODATIV"];
            array_push($rowsUpdate, $this->verificaCamposAlteradosProdfili($row, $produto));
            $count++;
            if($count >= 500){
                $dataSet = new DataSet($dataSourceNameToSave, $rowsUpdate);
                $this->dataSourceManager->persist($dataSet);
                $rowsUpdate = array();
                $count = 0;
            }
        }
        
        // var_dump($rowsUpdate);die;
        $dataSet = new DataSet($dataSourceNameToSave, $rowsUpdate);
        $this->dataSourceManager->persist($dataSet);
    }
    
    protected function getProdFili($CDFILIAL, $P_CDPRODINI, $P_CDPRODFIN, $TODOS) {
        $conditions =  array();
        $filterCriteria = new FilterCriteria('prodporfili_produto', $conditions);
        $paramsProdutos = $params = array(
                    ':CDFILIAL'=>$CDFILIAL,
                    ':P_CDPRODFIN'=>$P_CDPRODFIN,
                    ':P_CDPRODINI'=>$P_CDPRODINI,
                    ':TODOS'=>$TODOS
                );
        $produtos = $this->factory->getProdFili($params);
        $prodReturn = array();
        foreach ($produtos as $prod) {
            $produtoComLocal = $this->adicionaCampos($prod);
            array_push($prodReturn, $produtoComLocal);
        }
        return $prodReturn;
    }
    
    public function atualizaProdFili($row) {
        $boParam = false;
        $parametros = array("1" => "N", "2" => "N", "3" => "N", "4" => "N", "5" => "N", "6" => "N", "7" => "N",
                            "8" => "N", "9" => "N", "10" => "N", "11" => "N", "12" => "N", "13" => "N", "14" => "N");
                            
        if(count($row["NMLISTAPARAM"]) == 14 || $row["IDTPCONFPAR"] == "1" || $row["IDTPCONFPAR"] == "2") {
            $stringParametros = 'S';
            $boParam = true;
        }
        else {
            $stringParametros = 'N';
        }
        
        if($stringParametros == "N") {
            foreach($row["CDLISTAPARAM"] as $param) {
                $parametros[$param] = "S";
            }
            
            foreach($parametros as $param) {
                $stringParametros .= $param;
            }
        }
        
        if(count($row["CDFILIDEST"]) >  1) {
            $idTpFiliDest = 'L';
            $cdFiliDest   = '_';
            foreach($row["CDFILIDEST"] as $filial) {
                $cdFiliDest.=$filial.'_';
            }
        }
        else if(count($row["CDFILIDEST"]) == 1 && $row["CDFILIDEST"] != null) {
            $idTpFiliDest = 'F';
            $cdFiliDest = $row["CDFILIDEST"][0];
        }
        else {
            $idTpFiliDest = 'G';
            $cdFiliDest = $row["CDGRUPFILI"];
        }
        
        
        $this->factory->callProcedureAtualizaProdFili($row["CDFILIORIG"], $cdFiliDest, $idTpFiliDest, $row["IDTPCONFPAR"], $row["P_CDPRODINI"], 
                                            $row["P_CDPRODFIN"], $stringParametros);
                                            
        if($boParam == true || $parametros["3"] == "S" || $parametros["4"] == "S") {
            $this->deletaProd($idTpFiliDest, $cdFiliDest, $row["P_CDPRODINI"], $row["P_CDPRODFIN"]);
            if($idTpFiliDest == "G") {
                $filiais = $this->factory->getFiliaisGrupFili($row["CDGRUPFILI"]);
            }
            else $filiais = $row["CDFILIDEST"];
            
            foreach($filiais as $filial) {
                $cdFilial = (isset($filial["CDFILIAL"]) ? $filial["CDFILIAL"] : $filial);
                $prodControlEstoque = $this->factory->getProdControlEstoque($cdFilial, $row["P_CDPRODINI"], $row["P_CDPRODFIN"]);
               
                if(isset($prodControlEstoque[0])) {
                    try {
                        $nrLancto = $this->newCode->getCodeByTableName('RECALESTOQ', $this->environment->getNrOrgTrab(), 1, 10);
                
                        $this->factory->insereMoviLote($cdFilial,  $row["P_CDPRODINI"], $row["P_CDPRODFIN"], $nrLancto, $this->environment->getNrOrgTrab());
                        $this->factory->callProcedureAtualizaEstoque($cdFilial, date("d/m/Y"), $nrLancto);
                        $this->factory->deletaMoviLote($nrLancto);
                    }
                    catch(\Exception $e) {
                        $this->factory->deletaMoviLote($nrLancto);
                        throw($e);
                    }
                } 
            }
        }
    }
    
    public function deletaProd($idTpFiliDest, $cdFiliDest, $cdProdIni, $cdProdFin) {
        if($idTpFiliDest == "F") {
            $this->factory->deletaPosEstDiaFil($cdFiliDest, $cdProdIni, $cdProdFin);
            $this->factory->deletaPosEstConsFil($cdFiliDest, $cdProdIni, $cdProdFin);
            $this->factory->deletaPosEstAtuFil($cdFiliDest, $cdProdIni, $cdProdFin);
        }
        else if($idTpFiliDest == "G") {
            $this->factory->deletaPosEstDiaGrupFil($cdFiliDest, $cdProdIni, $cdProdFin);
            $this->factory->deletaPosEstConsGrupFil($cdFiliDest, $cdProdIni, $cdProdFin);
            $this->factory->deletaPosEstAtuGrupFil($cdFiliDest, $cdProdIni, $cdProdFin);
        }
        else {
            $this->factory->deletaPosEstDiaLstFil($cdFiliDest, $cdProdIni, $cdProdFin);
            $this->factory->deletaPosEstConsLstFil($cdFiliDest, $cdProdIni, $cdProdFin);
            $this->factory->deletaPosEstAtuLstFil($cdFiliDest, $cdProdIni, $cdProdFin);
        }
    }
    
    public function vardump($string){
        return $string;
    }
    
}